/*********************************************************************************************************************************
 * Autorskie Prawa Majątkowe - Moose Spółka z ograniczoną odpowiedzialnością
 *
 * Copyright 2017 Moose Spółka z ograniczoną odpowiedzialnością
 ********************************************************************************************************************************/

package eu.mooseinc.dell.form.web.controller;

import eu.mooseinc.dell.form.entity.MembershipType;
import eu.mooseinc.dell.form.entity.Registration;
import eu.mooseinc.dell.form.service.CaptchaService;
import eu.mooseinc.dell.form.service.MembershipTypeService;
import eu.mooseinc.dell.form.service.RegistrationService;
import eu.mooseinc.dell.form.util.FileFormUtils;
import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.List;
import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.servlet.http.HttpSession;
import javax.validation.Valid;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

/**
 * Registration form controller.
 */
@Controller
@Scope("session")
public class RegistrationFormController implements Serializable {
	private static final Logger logger = LoggerFactory.getLogger(RegistrationFormController.class);

	@Autowired
	private RegistrationService registrationService;

	@Autowired
	private MembershipTypeService membershipTypeService;

	@Autowired
	private CaptchaService captchaService;

	@Autowired
	private HttpSession session;

	@Value("${temporary.directory}")
	private String tempDir;

	@Value("${captcha.site_key}")
	private String captchaSiteKey;

	/**
	 * Session directory.
	 */
	private String sessionDirectoryName;

	/**
	 * Constructor.
	 */
	public RegistrationFormController() {
		super();
	}

	/**
	 * Initiation method for controller.
	 */
	@PostConstruct
	public void init() {
		this.sessionDirectoryName = FileFormUtils.getSessionDirectoryName(tempDir, session.getId());
		createSessionDirectory();
	}

	/**
	 * Method called before controller destroyed.
	 */
	@PreDestroy
	public void close() {
		deleteSessionDirectory();
	}

	/**
	 * Returns list of membership types.
	 *
	 * @return list of membership types.
	 */
	@ModelAttribute("allMembershipTypes")
	public List<MembershipType> getAllMembershipTypes() {
		return membershipTypeService.getMembershipTypes();
	}

	@ModelAttribute("siteKey")
	public String getSiteKey() {
		return captchaSiteKey;
	}


	/**
	 * Shows registration form.
	 *
	 * @param registration wregistration model.
	 *
	 * @return registration form view.
	 */
	@RequestMapping(value = {"/", "registrationForm"}, method = RequestMethod.GET)
	public String showContactForm(final Registration registration, final Model model) throws Exception {
		model.addAttribute("captcha", true);
		FileFormUtils.cleanSessionDirectory(sessionDirectoryName);
		return "registrationForm";
	}

	/**
	 * On sending registration data.
	 *
	 * @param registration registration data.
	 * @param bindingResult response resoult.
	 *
	 * @return confirmation view.
	 */
	@RequestMapping(value = "/", method = RequestMethod.POST, params = {"send"})
	public String sendMessage(final @Valid Registration registration,
			final @RequestParam(name = "g-recaptcha-response") String captcha,
			final BindingResult bindingResult,
			final RedirectAttributes attributes, final Model model) throws Exception {
		if (bindingResult.hasErrors())
			return "registrationForm";
		final boolean valid = captchaService.isCaptchaValid(captcha);
		if (!valid) {
			model.addAttribute("captcha", false);
			return "notificationForm";
		}
		registrationService.sendRegistration(registration);
		FileFormUtils.cleanSessionDirectory(sessionDirectoryName);
		return "redirect:/confirmation";
	}

	/**
	 * Creates session directory.
	 */
	private void createSessionDirectory() {
		try {
			final File sessionDirectory = new File(sessionDirectoryName);
			FileUtils.forceMkdir(sessionDirectory);
		}
		catch (final IOException e) {
			logger.error("Error with creating session directory", e);
		}
	}

	/**
	 * Usuwa katalog sesji.
	 */
	private void deleteSessionDirectory() {
		try {
			final File sessionDirectory = new File(sessionDirectoryName);
			FileUtils.deleteDirectory(sessionDirectory);
		}
		catch (final IOException e) {
			logger.error("Error with removing session directory", e);
		}
	}
}
